/* cc -O2 -o hspline hspline.c -lm
 */
#include "/home/jiri/gen/include.h"
#include "/home/jiri/gen/powi.c"

#define MAXORDER 12 /* max poly order */
typedef struct poly_s {
  int order;
  double coeff[MAXORDER+1]; } poly_t;  

double mypoly(poly_t *p,double x)
{
  int i=p->order;
  double sum=p->coeff[i];

  while (i--) sum=sum*x+p->coeff[i];

  return sum;
}

#define NSPLINE 80

poly_t a[NSPLINE],b[NSPLINE],c[NSPLINE],d[NSPLINE];
double arho[NSPLINE],brho[NSPLINE],crho[NSPLINE],drho[NSPLINE];

double node[NSPLINE+1];

#include "tailderi.c"

int main(int narg,char **arg)
{
  int i;
  double rho,r;

  if (narg<3) {
    fprintf(stderr,"Call by:\n\
  hspline NAME RHO\n\
uses NAME*.fit containing coeff of poly fit of splines, by 1/16, to r=6,\n\
divided by rho^int(r)\n\
");
    exit(0); }

  rho=atof(arg[2]);

  loop (i,0,NSPLINE) {
    char *fn=string("%s%d.fit",arg[1],i);
    FILE *f=fopen(fn,"rt");
    int j,jj,o;
    char line[1024];

    if (!f) Error(fn);
    loopto (j,0,MAXORDER) if (fgets(line,1024,f)) if (line[0]!='#')
      if (5==sscanf(line,"%d%lf%lf%lf%lf",
		    &jj,a[i].coeff+j,b[i].coeff+j,c[i].coeff+j,d[i].coeff+j)) {
	o=jj;
	if (jj!=j) Error("bad index in lineX.fit"); }

    a[i].order=b[i].order=c[i].order=d[i].order=o; }
    
  loopto (i,0,NSPLINE) node[i]=1+i/16.;

  /* refine c,d to recover smoothness - must be done for given rho */
  {
    double ax=tail(rho,node[NSPLINE]);
    double bx=tailder(rho,node[NSPLINE]);
    double r1,r2,h;
    int ispline;

    /* this is necessary only at points of change of the order, 
       i.e., r=integer 
       but here done for all... */

    for (ispline=NSPLINE-1; ispline>=0; ispline--) {
      int rhopow=ispline/16+1;
      double rp=powi(rho,rhopow);
 
      arho[ispline]=mypoly(a+ispline,rho)*rp;
      brho[ispline]=mypoly(b+ispline,rho)*rp;
#if 0
      crho[ispline]=mypoly(c+ispline,rho)*rp;
      drho[ispline]=mypoly(d+ispline,rho)*rp;
      put3(ispline,crho[ispline],drho[ispline])
#endif

      h=node[ispline+1]-node[ispline]; /* now only 1./16 */
      r1=(ax-arho[ispline]-brho[ispline]*h)/(h*h);
      r2=(bx-brho[ispline])/h;
      crho[ispline]=3*r1-r2;
      drho[ispline]=(r2-2*r1)/h;
#if 0
      put3(0,crho[ispline],drho[ispline])
#endif
      ax=arho[ispline];
      bx=brho[ispline]; } }    

  for (r=1.0005; r<10; r+=0.001) {
    double hhh;

    if (r>node[NSPLINE])
      hhh=tail(rho,r);
    else {
      int ispline=(int)((r-1)*16.);
      double dr=r-node[ispline];
#if 0
      /* c,d not refined */
      int rhopow=ispline/16+1;
      double rp=powi(rho,rhopow);
      double A=mypoly(a+ispline,rho)*rp;
      double B=mypoly(b+ispline,rho)*rp;
      double C=mypoly(c+ispline,rho)*rp;
      double D=mypoly(d+ispline,rho)*rp;
      hhh=A+dr*(B+dr*(C+dr*D));
#else
      /* c,d not refined */
      hhh=arho[ispline]+dr*(brho[ispline]+dr*(crho[ispline]+dr*drho[ispline])); 
#endif
      }

    printf("%.4f %g\n",r,hhh); }

  return 0;
}
